/* Copyright (C) 2012 Chris Vine

The library comprised in this file or of which this file is part is
distributed by Chris Vine under the GNU Lesser General Public
License as follows:

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License, version 2.1, for more details.

   You should have received a copy of the GNU Lesser General Public
   License, version 2.1, along with this library (see the file LGPL.TXT
   which came with this source code package in the c++-gtk-utils
   sub-directory); if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <glib.h>
#include <string>
#include <memory>

#include <c++-gtk-utils/async_queue.h>
#include <c++-gtk-utils/thread.h>

using namespace Cgu;

extern "C" {
static void test_async_queue_simple() {
  AsyncQueue<std::string> q;

  q.push(std::string("Hello"));
  q.emplace("there");

  g_assert_cmpuint((unsigned int)q.empty(), ==, 0);
  std::string s;
  q.pop(s);
  g_assert_cmpstr(s.c_str(), ==, "Hello");
  q.pop(s);
  g_assert_cmpstr(s.c_str(), ==, "there");
  g_assert_cmpuint((unsigned int)q.empty(), !=, 0);
}

static void test_async_queue_dispatch() {
  AsyncQueueDispatch<std::string> q;

  q.push(std::string("Hello"));
  q.emplace("there");

  g_assert_cmpuint((unsigned int)q.empty(), ==, 0);
  std::string s;
  q.pop_dispatch(s);
  g_assert_cmpstr(s.c_str(), ==, "Hello");
  q.pop_dispatch(s);
  g_assert_cmpstr(s.c_str(), ==, "there");
  g_assert_cmpuint((unsigned int)q.empty(), !=, 0);

  // gcc-4.4 doesn't support lambdas, use a struct in function scope
  struct ThreadFunc {
    static void func(AsyncQueueDispatch<std::string>& q) {
      q.push(std::string("thread"));
    }
  };
  std::unique_ptr<Thread::Thread> t(
    Thread::Thread::start(Callback::make<AsyncQueueDispatch<std::string>&>(&ThreadFunc::func,
									   q),
			  false));
  g_assert(t.get() != static_cast<Thread::Thread*>(0));

  q.pop_dispatch(s);
  g_assert_cmpstr(s.c_str(), ==, "thread");
}

} // extern "C"


int main (int argc, char* argv[]) {
  g_test_init(&argc, &argv, static_cast<void*>(0));

  g_test_add_func("/async_queue/simple", test_async_queue_simple); 
  g_test_add_func("/async_queue/dispatch", test_async_queue_dispatch); 

  return g_test_run();
}
