/* Copyright (C) 2012 Chris Vine

The library comprised in this file or of which this file is part is
distributed by Chris Vine under the GNU Lesser General Public
License as follows:

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License, version 2.1, for more details.

   You should have received a copy of the GNU Lesser General Public
   License, version 2.1, along with this library (see the file LGPL.TXT
   which came with this source code package in the c++-gtk-utils
   sub-directory); if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <glib.h>
#include <vector>

#include <c++-gtk-utils/io_watch.h>
#include <c++-gtk-utils/timeout.h>
#include <c++-gtk-utils/callback.h>
#include <c++-gtk-utils/thread.h>
#include <c++-gtk-utils/emitter.h>
#include <c++-gtk-utils/pipes.h>

using namespace Cgu;

class Test {
  int count;
public:
  Releaser releaser;
  void iowatch_func(GIOCondition, bool&);
  Test(): count(0) {}
};

Test* test;
GMainLoop* loop;
PipeFifo p{PipeFifo::block};
std::vector<unsigned char> result;

// this executes in the main thread
void Test::iowatch_func(GIOCondition c, bool& keep) {
  g_assert(c & G_IO_IN);
  ++count;
  g_assert_cmpint(count, <=, 3);
  result.push_back(p.read());
  
  if (count == 3) delete test; // Releaser will remove the watch callback
                               // so we won't get the last two characters
}

// this executes in the main thread
void timeout_func(bool& keep) {
  if (result.size() < 3) return;
  g_main_loop_quit(loop);
  keep = false;
}

void thread_func() {
  int count = 0;
  char letter = 'a';
  for (; count < 5; ++count, ++letter) {
    p.write(letter);
  }
}

extern "C" {
static void test_iowatch() {

  test = new Test;
  loop = g_main_loop_new(0, false);

  start_iowatch(p.get_read_fd(),
                Callback::make(*test, &Test::iowatch_func),
		test->releaser,
                GIOCondition(G_IO_IN | G_IO_HUP | G_IO_ERR));

  Thread::JoinableHandle t{Thread::Thread::start(Callback::make(thread_func), true),
                           Thread::JoinableHandle::join_on_exit};
  start_timeout(300, Callback::make(&timeout_func), G_PRIORITY_LOW);

  g_main_loop_run(loop);
  t.join();

  g_assert_cmpuint(result.size(), ==, 3);
  g_assert_cmpuint(result[0], ==, (unsigned int)'a');
  g_assert_cmpuint(result[1], ==, (unsigned int)'b');
  g_assert_cmpuint(result[2], ==, (unsigned int)'c');
}
} // extern "C"


int main (int argc, char* argv[]) {
  g_test_init(&argc, &argv, static_cast<void*>(0));

  g_test_add_func("/iowatch/iowatch", test_iowatch); 

  return g_test_run();
}
